@@ -21,8 +21,10 @@ import (
 	"context"
 	"go/types"
 	"log"
+	"os"
 	"path"
 	"reflect"
+	"strings"
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
@@ -54,6 +56,11 @@ type Config struct {
 	// If Dir is empty, the tool is run in the current directory.
 	Dir string
 
+	// WorkingDir is the directory in which to run gop compiler.
+	// TargetDir is the directory in which to generate Go files.
+	// If SrcDir or TargetDir is empty, it is same as Dir.
+	WorkingDir, TargetDir string
+
 	// Env is the environment to use when invoking the build system's query tool.
 	// If Env is nil, the current environment is used.
 	// As in os/exec's Cmd, only the last value in the slice for
@@ -68,8 +75,31 @@ type Config struct {
 	// the build system's query tool.
 	BuildFlags []string
 
-	// LoadPkgs is called to load all import packages.
-	LoadPkgs gox.LoadPkgsFunc
+	// Fset provides source position information for syntax trees and types.
+	// If Fset is nil, Load will use a new fileset, but preserve Fset's value.
+	Fset *token.FileSet
+
+	// GenGoPkg is called to convert a Go+ package into Go.
+	GenGoPkg func(pkgDir string, base *Config) error
+
+	// PkgsLoader is the Go+ packages loader (will be set if it is nil).
+	PkgsLoader *PkgsLoader
+
+	// CacheLoadPkgs set to cache all loaded packages or not.
+	CacheLoadPkgs bool
+
+	// NoFileLine = true means not generate file line comments.
+	NoFileLine bool
+}
+
+func (conf *Config) Ensure() *Config {
+	if conf == nil {
+		conf = &Config{Fset: token.NewFileSet()}
+	}
+	if conf.PkgsLoader == nil {
+		initPkgsLoader(conf)
+	}
+	return conf
 }
 
 func getUnderlying(ctx *blockCtx, typ types.Type) types.Type {
@@ -83,45 +113,69 @@ func getUnderlying(ctx *blockCtx, typ types.Type) types.Type {
 }
 
 func doLoadUnderlying(ctx *pkgCtx, typ *types.Named) types.Type {
-	panic("TODO: doLoadUnderlying")
-	/*
-		ld := ctx.syms[typ.Obj().Name()].(*typeLoader)
-		doInitType(ld)
-		return typ.Underlying()
-	*/
+	ld := ctx.syms[typ.Obj().Name()].(*typeLoader)
+	ld.load()
+	return typ.Underlying()
+}
+
+type nodeInterp struct {
+	fset       *token.FileSet
+	files      map[string]*ast.File
+	workingDir string
+}
+
+func (p *nodeInterp) Position(pos token.Pos) token.Position {
+	return p.fset.Position(pos)
+}
+
+func (p *nodeInterp) LoadExpr(node ast.Node) (src string, pos token.Position) {
+	start := node.Pos()
+	pos = p.fset.Position(start)
+	f := p.files[pos.Filename]
+	n := int(node.End() - start)
+	pos.Filename = relFile(p.workingDir, pos.Filename)
+	src = string(f.Code[pos.Offset : pos.Offset+n])
+	return
 }
 
 // NewPackage creates a Go+ package instance.
-func NewPackage(
-	pkgPath string, pkg *ast.Package, fset *token.FileSet, conf *Config) (p *gox.Package, err error) {
-	if conf == nil {
-		conf = &Config{}
+func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package, err error) {
+	conf = conf.Ensure()
+	dir := conf.Dir
+	if dir == "" {
+		dir, _ = os.Getwd()
+	}
+	workingDir := conf.WorkingDir
+	if workingDir == "" {
+		workingDir, _ = os.Getwd()
+	}
+	targetDir := conf.TargetDir
+	if targetDir == "" {
+		targetDir = dir
 	}
 	ctx := &pkgCtx{syms: make(map[string]loader)}
 	loadUnderlying := func(at *gox.Package, typ *types.Named) types.Type {
 		return doLoadUnderlying(ctx, typ)
 	}
-	loadPkgs := conf.LoadPkgs
-	if loadPkgs == nil {
-		loadPkgs = LoadGopPkgs
-	}
 	confGox := &gox.Config{
-		Context:        conf.Context,
-		Logf:           conf.Logf,
-		Dir:            conf.Dir,
-		Env:            conf.Env,
-		BuildFlags:     conf.BuildFlags,
-		Fset:           fset,
-		ParseFile:      nil, // TODO
-		LoadPkgs:       loadPkgs,
-		LoadUnderlying: loadUnderlying,
-		Prefix:         gopPrefix,
-		Contracts:      nil,
-		NewBuiltin:     newBuiltinDefault,
+		Context:         conf.Context,
+		Logf:            conf.Logf,
+		Dir:             dir,
+		Env:             conf.Env,
+		BuildFlags:      conf.BuildFlags,
+		Fset:            conf.Fset,
+		LoadPkgs:        conf.PkgsLoader.LoadPkgs,
+		LoadUnderlying:  loadUnderlying,
+		HandleErr:       ctx.handleErr,
+		NodeInterpreter: &nodeInterp{fset: conf.Fset, files: pkg.Files, workingDir: workingDir},
+		Prefix:          gopPrefix,
+		ParseFile:       nil, // TODO
+		NewBuiltin:      newBuiltinDefault,
 	}
+	fileLine := !conf.NoFileLine
 	p = gox.NewPackage(pkgPath, pkg.Name, confGox)
 	for _, f := range pkg.Files {
-		loadFile(p, ctx, f)
+		loadFile(p, ctx, f, targetDir, fileLine)
 	}
 	for _, f := range pkg.Files {
 		for _, decl := range f.Decls {
@@ -144,7 +198,10 @@ func NewPackage(
 			}
 		}
 	}
-	ctx.complete()
+	for _, load := range ctx.inits {
+		load()
+	}
+	err = ctx.complete()
 	return
 }
 
@@ -201,18 +258,43 @@ func doInitMethods(ld *typeLoader) {
 	}
 }
 
+type Errors struct {
+	Errs []error
+}
+
+func (p *Errors) Error() string {
+	msgs := make([]string, len(p.Errs))
+	for i, err := range p.Errs {
+		msgs[i] = err.Error()
+	}
+	return strings.Join(msgs, "\n")
+}
+
 type pkgCtx struct {
-	syms map[string]loader
+	syms  map[string]loader
+	inits []func()
+	errs  []error
 }
 
 type blockCtx struct {
 	*pkgCtx
-	pkg     *gox.Package
-	cb      *gox.CodeBuilder
-	imports map[string]*gox.PkgRef
+	pkg       *gox.Package
+	cb        *gox.CodeBuilder
+	fset      *token.FileSet
+	targetDir string
+	fileLine  bool
+	imports   map[string]*gox.PkgRef
 }
 
-func (p *pkgCtx) complete() {
+func (p *pkgCtx) handleErr(err error) {
+	p.errs = append(p.errs, err)
+}
+
+func (p *pkgCtx) complete() error {
+	if p.errs != nil {
+		return &Errors{Errs: p.errs}
+	}
+	return nil
 }
 
 func (p *pkgCtx) loadType(name string) {
@@ -236,22 +318,25 @@ func (p *pkgCtx) loadSymbol(name string) bool {
 	return false
 }
 
-func loadFile(p *gox.Package, parent *pkgCtx, f *ast.File) {
+func loadFile(p *gox.Package, parent *pkgCtx, f *ast.File, targetDir string, fileLine bool) {
 	syms := parent.syms
 	ctx := &blockCtx{
-		pkg: p, pkgCtx: parent, cb: p.CB(), imports: make(map[string]*gox.PkgRef),
+		pkg: p, pkgCtx: parent, cb: p.CB(), fset: p.Fset, targetDir: targetDir, fileLine: fileLine,
+		imports: make(map[string]*gox.PkgRef),
 	}
 	for _, decl := range f.Decls {
 		switch d := decl.(type) {
 		case *ast.FuncDecl:
 			if d.Recv == nil {
 				name := d.Name.Name
-				if name == "init" {
-					log.Panicln("loadFunc TODO: init")
-				}
-				syms[name] = loaderFunc(func() {
+				fn := func() {
 					loadFunc(ctx, nil, d)
-				})
+				}
+				if name == "init" {
+					parent.inits = append(parent.inits, fn)
+				} else {
+					syms[name] = loaderFunc(fn)
+				}
 			} else {
 				name := getRecvTypeName(d.Recv)
 				ld := getTypeLoader(syms, name)
@@ -321,7 +406,7 @@ func loadType(ctx *blockCtx, t *ast.TypeSpec) {
 
 func loadFunc(ctx *blockCtx, recv *types.Var, d *ast.FuncDecl) {
 	sig := toFuncType(ctx, d.Type, recv)
-	fn := ctx.pkg.NewFuncWith(d.Name.Name, sig)
+	fn := ctx.pkg.NewFuncWith(d.Pos(), d.Name.Name, sig)
 	if body := d.Body; body != nil {
 		loadFuncBody(ctx, fn, body)
 	}
