@@ -30,32 +30,30 @@ import (
 )
 
 var (
-	gblFset     *token.FileSet
-	gblLoadPkgs gox.LoadPkgsFunc
+	gblFset  *token.FileSet
+	baseConf *cl.Config
 )
 
 func init() {
 	gox.SetDebug(gox.DbgFlagAll)
 	gblFset = token.NewFileSet()
-	gblLoadPkgs = gox.NewLoadPkgsCached(cl.LoadGop)
-	cl.GenGoPkg = gengo.NewRunner(nil, nil).GenGoPkg
+	baseConf = &cl.Config{
+		Fset:          gblFset,
+		GenGoPkg:      new(gengo.Runner).GenGoPkg,
+		CacheLoadPkgs: true,
+		NoFileLine:    true,
+	}
 }
 
-func gopClTest(t *testing.T, gopcode, expected string, nocache ...bool) {
-	fset := gblFset
+func gopClTest(t *testing.T, gopcode, expected string) {
 	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", gopcode)
-	pkgs, err := parser.ParseFSDir(fset, fs, "/foo", nil, 0)
+	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
 	if err != nil {
 		t.Fatal("ParseFSDir:", err)
 	}
-	conf := &cl.Config{
-		LoadPkgs: gblLoadPkgs,
-	}
-	if nocache != nil {
-		conf = nil
-	}
+	conf := *baseConf.Ensure()
 	bar := pkgs["main"]
-	pkg, err := cl.NewPackage("", bar, fset, conf)
+	pkg, err := cl.NewPackage("", bar, &conf)
 	if err != nil {
 		t.Fatal("NewPackage:", err)
 	}
@@ -70,6 +68,29 @@ func gopClTest(t *testing.T, gopcode, expected string, nocache ...bool) {
 	}
 }
 
+func TestNilConf(t *testing.T) {
+	fs := parsertest.NewSingleFileFS("/foo", "bar.gop", `println("Hi")`)
+	pkgs, err := parser.ParseFSDir(gblFset, fs, "/foo", nil, 0)
+	if err != nil {
+		t.Fatal("ParseFSDir:", err)
+	}
+	cl.NewPackage("", pkgs["main"], nil)
+}
+
+func TestInitFunc(t *testing.T) {
+	gopClTest(t, `
+
+func init() {}
+func init() {}
+`, `package main
+
+func init() {
+}
+func init() {
+}
+`)
+}
+
 func TestSelect(t *testing.T) {
 	gopClTest(t, `
 
@@ -467,10 +488,18 @@ var z builtin.Gop_bigint = x.Gop_Add(y)
 
 func TestTypeConv(t *testing.T) {
 	gopClTest(t, `
+var a = (*struct{})(nil)
+var b = interface{}(nil)
+var c = (func())(nil)
 var x uint32 = uint32(0)
 var y *uint32 = (*uint32)(nil)
 `, `package main
 
+var a = (*struct {
+})(nil)
+var b = interface {
+}(nil)
+var c = (func())(nil)
 var x uint32 = uint32(0)
 var y *uint32 = (*uint32)(nil)
 `)
@@ -1333,7 +1362,7 @@ func TestUnnamedMainFunc(t *testing.T) {
 func main() {
 	i := 1
 }
-`, true)
+`)
 }
 
 func TestFuncAsParam(t *testing.T) {
