@@ -80,6 +80,8 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 		ctx.cb.Typ(toStructType(ctx, v))
 	case *ast.ChanType:
 		ctx.cb.Typ(toChanType(ctx, v))
+	case *ast.InterfaceType:
+		ctx.cb.Typ(toInterfaceType(ctx, v))
 	case *ast.ComprehensionExpr:
 		compileComprehensionExpr(ctx, v, twoValue != nil && twoValue[0])
 	case *ast.TypeAssertExpr:
@@ -88,6 +90,8 @@ func compileExpr(ctx *blockCtx, expr ast.Expr, twoValue ...bool) {
 		compileExpr(ctx, v.X)
 	case *ast.ErrWrapExpr:
 		compileErrWrapExpr(ctx, v)
+	case *ast.FuncType:
+		ctx.cb.Typ(toFuncType(ctx, v, nil))
 	case *ast.Ellipsis:
 		panic("compileEllipsis: ast.Ellipsis unexpected")
 	case *ast.KeyValueExpr:
@@ -113,7 +117,7 @@ func compileUnaryExpr(ctx *blockCtx, v *ast.UnaryExpr, twoValue bool) {
 func compileBinaryExpr(ctx *blockCtx, v *ast.BinaryExpr) {
 	compileExpr(ctx, v.X)
 	compileExpr(ctx, v.Y)
-	ctx.cb.BinaryOp(gotoken.Token(v.Op))
+	ctx.cb.BinaryOp(gotoken.Token(v.Op), v)
 }
 
 func compileIndexExprLHS(ctx *blockCtx, v *ast.IndexExpr) {
@@ -191,22 +195,24 @@ func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, autoCall bool) {
 	default:
 		compileExpr(ctx, v.X)
 	}
-	var mflag int
-	var name = v.Sel.Name
-	cb.MemberVal(name, &mflag)
-	if mflag == 0 {
-		if c := name[0]; c >= 'a' && c <= 'z' {
-			name = string(rune(c)+('A'-'a')) + name[1:]
-			cb.MemberVal(name, &mflag)
-			if mflag == gox.MFlagMethod {
-				if autoCall {
-					cb.Call(0)
-				}
-				return
-			}
-		}
-		log.Panicln("TODO: member not found -", v.Sel.Name)
+	name := v.Sel.Name
+	kind, err := cb.Member(name, v)
+	if kind != 0 {
+		return
 	}
+	if c := name[0]; c >= 'a' && c <= 'z' {
+		name = string(rune(c)+('A'-'a')) + name[1:]
+		switch kind, _ = cb.Member(name, v); kind {
+		case gox.MemberMethod:
+			if autoCall {
+				cb.Call(0)
+			}
+			return
+		case gox.MemberField:
+			return
+		}
+	}
+	panic(err)
 }
 
 func compileCallExpr(ctx *blockCtx, v *ast.CallExpr) {
@@ -225,10 +231,13 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr) {
 }
 
 func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
+	cb := ctx.cb
+	comments := cb.Comments()
 	sig := toFuncType(ctx, v.Type, nil)
-	fn := ctx.cb.NewClosureWith(sig)
+	fn := cb.NewClosureWith(sig)
 	if body := v.Body; body != nil {
 		loadFuncBody(ctx, fn, body)
+		cb.SetComments(comments, false)
 	}
 }
 
@@ -255,7 +264,7 @@ func compileIdent(ctx *blockCtx, ident *ast.Ident, allowBuiltin bool) {
 				panic("unexpected builtin: " + name)
 			}
 		}
-		ctx.cb.Val(o)
+		ctx.cb.Val(o, ident)
 	} else {
 		log.Panicln("TODO: var not found -", name)
 	}
@@ -285,7 +294,7 @@ func compileBasicLit(ctx *blockCtx, v *ast.BasicLit) {
 	ctx.cb.Val(&goast.BasicLit{
 		Kind:  gotoken.Token(v.Kind),
 		Value: v.Value,
-	})
+	}, v)
 }
 
 const (
