@@ -17,15 +17,40 @@
 package cl
 
 import (
+	"fmt"
 	"log"
+	"path/filepath"
 	"reflect"
 
+	goast "go/ast"
 	gotoken "go/token"
 
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
 )
 
+func relFile(dir string, file string) string {
+	if rel, err := filepath.Rel(dir, file); err == nil {
+		if rel[0] == '.' {
+			return rel
+		}
+		return "./" + rel
+	}
+	return file
+}
+
+func commentStmt(ctx *blockCtx, stmt ast.Stmt) {
+	if ctx.fileLine {
+		start := stmt.Pos()
+		pos := ctx.fset.Position(start)
+		line := fmt.Sprintf("\n//line %s:%d", relFile(ctx.targetDir, pos.Filename), pos.Line)
+		comments := &goast.CommentGroup{
+			List: []*goast.Comment{{Text: line}},
+		}
+		ctx.cb.SetComments(comments, false)
+	}
+}
+
 func compileStmts(ctx *blockCtx, body []ast.Stmt) {
 	for _, stmt := range body {
 		compileStmt(ctx, stmt)
@@ -33,6 +58,17 @@ func compileStmts(ctx *blockCtx, body []ast.Stmt) {
 }
 
 func compileStmt(ctx *blockCtx, stmt ast.Stmt) {
+	defer func() {
+		if e := recover(); e != nil {
+			if err, ok := e.(error); ok {
+				ctx.handleErr(err)
+				ctx.cb.ResetStmt()
+			} else {
+				panic(e)
+			}
+		}
+	}()
+	commentStmt(ctx, stmt)
 	switch v := stmt.(type) {
 	case *ast.ExprStmt:
 		compileExpr(ctx, v.X)
@@ -109,6 +145,12 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 			}
 		}
 		ctx.cb.DefineVarStart(names...)
+		defer func() {
+			if e := recover(); e != nil {
+				ctx.cb.ResetInit()
+				panic(e)
+			}
+		}()
 		for _, rhs := range expr.Rhs {
 			compileExpr(ctx, rhs, twoValue)
 		}
@@ -139,6 +181,7 @@ func compileAssignStmt(ctx *blockCtx, expr *ast.AssignStmt) {
 // end
 func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	if v.Tok == token.DEFINE {
 		names := make([]string, 1, 2)
 		if v.Key == nil {
@@ -171,11 +214,13 @@ func compileRangeStmt(ctx *blockCtx, v *ast.RangeStmt) {
 	}
 	cb.RangeAssignThen()
 	compileStmts(ctx, v.Body.List)
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
 func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	names := make([]string, 1, 2)
 	if v.Key == nil {
 		names[0] = "_"
@@ -193,10 +238,12 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 		compileExpr(ctx, v.Cond)
 		cb.Then()
 		compileStmts(ctx, v.Body.List)
+		cb.SetComments(comments, true)
 		cb.End()
 	} else {
 		compileStmts(ctx, v.Body.List)
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
@@ -206,6 +253,7 @@ func compileForPhraseStmt(ctx *blockCtx, v *ast.ForPhraseStmt) {
 // end
 func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	cb.For()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -221,6 +269,7 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 		cb.Post()
 		compileStmt(ctx, v.Post)
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
@@ -229,6 +278,7 @@ func compileForStmt(ctx *blockCtx, v *ast.ForStmt) {
 // end
 func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	cb.If()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -240,6 +290,7 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 		cb.Else()
 		compileStmt(ctx, v.Else)
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
@@ -253,6 +304,7 @@ func compileIfStmt(ctx *blockCtx, v *ast.IfStmt) {
 // end
 func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 	var cb = ctx.cb
+	comments := cb.Comments()
 	var name string
 	var ta *ast.TypeAssertExpr
 	switch stmt := v.Assign.(type) {
@@ -284,8 +336,10 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 		}
 		cb.TypeCase(len(c.List)) // TypeCase(0) means default case
 		compileStmts(ctx, c.Body)
+		commentStmt(ctx, stmt)
 		cb.End()
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
@@ -299,6 +353,7 @@ func compileTypeSwitchStmt(ctx *blockCtx, v *ast.TypeSwitchStmt) {
 // end
 func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	cb.Switch()
 	if v.Init != nil {
 		compileStmt(ctx, v.Init)
@@ -323,8 +378,10 @@ func compileSwitchStmt(ctx *blockCtx, v *ast.SwitchStmt) {
 		if has {
 			cb.Fallthrough()
 		}
+		commentStmt(ctx, stmt)
 		cb.End()
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
@@ -351,6 +408,7 @@ func hasFallthrough(body []ast.Stmt) ([]ast.Stmt, bool) {
 // end
 func compileSelectStmt(ctx *blockCtx, v *ast.SelectStmt) {
 	cb := ctx.cb
+	comments := cb.Comments()
 	cb.Select()
 	for _, stmt := range v.Body.List {
 		c, ok := stmt.(*ast.CommClause)
@@ -364,30 +422,33 @@ func compileSelectStmt(ctx *blockCtx, v *ast.SelectStmt) {
 		}
 		cb.CommCase(n) // CommCase(0) means default case
 		compileStmts(ctx, c.Body)
+		commentStmt(ctx, stmt)
 		cb.End()
 	}
+	cb.SetComments(comments, true)
 	cb.End()
 }
 
 func compileBranchStmt(ctx *blockCtx, v *ast.BranchStmt) {
+	label := v.Label
 	switch v.Tok {
 	case token.GOTO:
-		if v.Label == nil {
+		if label == nil {
 			log.Panicln("TODO: label not defined")
 		}
-		ctx.cb.Goto(v.Label.Name)
+		ctx.cb.Goto(label.Name, label)
 	case token.BREAK:
 		var name string
-		if v.Label != nil {
-			name = v.Label.Name
+		if label != nil {
+			name = label.Name
 		}
-		ctx.cb.Break(name)
+		ctx.cb.Break(name, label)
 	case token.CONTINUE:
 		var name string
-		if v.Label != nil {
-			name = v.Label.Name
+		if label != nil {
+			name = label.Name
 		}
-		ctx.cb.Continue(name)
+		ctx.cb.Continue(name, label)
 	case token.FALLTHROUGH:
 		panic("TODO: fallthrough statement out of place")
 	default:
@@ -396,7 +457,7 @@ func compileBranchStmt(ctx *blockCtx, v *ast.BranchStmt) {
 }
 
 func compileLabeledStmt(ctx *blockCtx, v *ast.LabeledStmt) {
-	ctx.cb.Label(v.Label.Name)
+	ctx.cb.Label(v.Label.Name, v.Label)
 	compileStmt(ctx, v.Stmt)
 }
 
